import subprocess
import os
import math
import netCDF4

def default_GMT():
    subprocess.run("gmtdefaults -D > .gmtdefaults4",shell=True)
    subprocess.run("gmtset TICK_LENGTH -0.075",shell=True)
    subprocess.run("gmtset TICK_PEN 1.5p",shell=True)
    subprocess.run("gmtset BASEMAP_TYPE plain",shell=True)
    subprocess.run("gmtset MEASURE_UNIT cm",shell=True)
#    subprocess.run("gmtset ANOT_FONT_SIZE_PRIMARY 13p",shell=True)
#    subprocess.run("gmtset ANOT_FONT_SIZE_SECONDARY 13p",shell=True)
    subprocess.run("gmtset LABEL_FONT_SIZE 16p",shell=True)
    subprocess.run("gmtset LABEL_OFFSET 0.0c",shell=True)
    subprocess.run("gmtset ELLIPSOID Sphere",shell=True)

def set_deg_per1km(lat):
    ny = len(lat)
    mean = 0.5*(lat[ny-1] + lat[0])
    Earth_RADIUS = 6378137.0
    return (360*1000)/(2*math.pi*(Earth_RADIUS * math.cos(mean*math.pi/180.0)))

def set_text_locate_top_center(slon,elon,slat,elat,mergin,fsiz,Tsiz=0):

    text_locate_centerX = str( 0.5*(slon + elon))
    text_locate_centerY = str( elat + mergin/fsiz*(elat-slat)) 
    text_locate = text_locate_centerX +" "+ text_locate_centerY 
    if(Tsiz != 0):
        text_locate += " " + str(Tsiz) + " 0 0 CB " 
    return text_locate

def set_text_locate_top_left(slon,elon,slat,elat,mergin,fsiz,Tsiz=0):

    text_locate_centerX = str(slon)
    text_locate_centerY = str( elat + mergin/fsiz*(elat-slat)) 
    text_locate = text_locate_centerX +" "+ text_locate_centerY 
    if(Tsiz != 0):
        text_locate += " " + str(Tsiz) + " 0 0 LB " 
    return text_locate

def set_text_locate_top_right(slon,elon,slat,elat,mergin,fsiz,Tsiz=0):

    text_locate_centerX = str(elon)
    text_locate_centerY = str( elat + mergin/fsiz*(elat-slat)) 
    text_locate = text_locate_centerX +" "+ text_locate_centerY 

    if(Tsiz != 0):
        text_locate += " " + str(Tsiz) + " 0 0 RB " 
    return text_locate

def set_text_locate_rightside_bottom(slon,elon,slat,elat,mergin,fsiz,Tsiz=0):

    text_locate_centerX = str( elon + mergin/fsiz*(elon-slon)) 
    text_locate_centerY = str(slat)
    text_locate = text_locate_centerX +" "+ text_locate_centerY 
    if(Tsiz != 0):
        text_locate += " " + str(Tsiz) + " 0 0 LB " 
    return text_locate

def set_scale_locate_bottom_right(fsizX,fsizY,mergeX,mergeY,height,width):
    xx = fsizX + mergeX
    yy = 0.0   + mergeY + height/2.0
    DD = "-D" + str(xx) + "/" + str(yy) +"/" + str(height) + "/" + str(width)
    return DD

def make_PNG(fsizX,fsizY,yohakuX,yohakuY,DPI,PS,PNG):
    
    # cm->inch Figure size will be expanded by yohaku cm , 1cm = 0.393701 inch
    Fsiz_inchX = (fsizX + yohakuX) * 0.393701
    Fsiz_inchY = (fsizY + yohakuY) * 0.393701

    X_pics = int(Fsiz_inchX * DPI)
    Y_pics = int(Fsiz_inchY * DPI)

    command = "gs " + "-dNOPAUSE -dBATCH -r" + str(DPI) + " -g" +\
              str(X_pics)+"x"+ str(Y_pics) + " -sOutputFile=" + PNG + " -sDEVICE=png16m" \
            +" -q " + PS
    print (command)
    subprocess.run(command,shell=True)


def auto_makecpt(shade,col_pattern,shade_col,slon,elon,slat,elat):
    nc = netCDF4.Dataset(shade,'r')
    lon = nc.variables["x"]
    lat = nc.variables["y"]
    A   = nc.variables["z"]

    max = -1000000
    min  =  1000000
    
    for (j,yy) in enumerate(lat):
        for (i,xx) in enumerate(lon):

            if xx >= slon and xx <= elon and yy >= slat and yy <= elat:
                if A[j][i] > max:
                    max = A[j][i]
                if A[j][i] < min:
                    min = A[j][i]

    nc.close()

    intv = (max - min)/ 6
    command = "makecpt " + "-C" + col_pattern + " -T"+str(min) \
              + "/" +str(max) + "/" + str(intv) + "> " + shade_col

#    print ("command=",command)
    subprocess.run(command,shell=True)


    return

def draw_GMT(shade="non",u="non",v="non",cont="non",cont_info="non",
             shade_col="non",
             slon=-999,slat=-999,elon=-999,lon_per_1km=0.0109664,
             fsizX=14,fsizY=14,
             PS="hoge.ps",PNG="non",bbase="nesw",
             JboundArea="non",
             vec_INTV=0.08,vec_info="0.03c/0.11c/0.1c -W3 -G0",
             title="non",unit="non",vec_per1cm=48,
             obj_draw_data=[[0,0],[1,1]],obj_draw_command="non",
             PNG_DPI = 100.0,PNG_yohakuX = 5,PNG_yohakuY = 2):

    if(slon==elon or slon > elon or slon < -180 or slat < -90 or elon < -180):
        print ("WRONG LAT/LON information input")
        exit(1)
        
    if(shade == "non"):
        print ("shade information must be set")
        exit(1)


    default_GMT()

    xlen = elon - slon
    ylen = xlen * 0.009/lon_per_1km * fsizY/fsizX
    elat = slat + ylen 
    rrr  = str(slon) + "/" +str(elon) + "/" +str(slat) + "/" +str(elat)

    if not os.path.exists(shade_col):
        print ("shade color palette was not given \n")
        auto_makecpt(shade,"no_green",shade_col,slon,elon,slat,elat)
        
    proj    = " -JX" + str(fsizX) + "/" + str(fsizY)
    rrange  = " -R" + rrr + " "
    ccol    = " -C" + shade_col + " "
    mmap    = rrange + proj + " -P "
    Bbase   = "-B" + bbase
    GMT_F   = " -K >" + PS
    GMT_M   = " -K -O >>" + PS
    GMT_E   = " -O >>" + PS


    # DRAW SHADE #####
    command = "grdimage " + shade + mmap +  ccol + "-Y1 " + GMT_F
    subprocess.run(command,shell=True)


    # DRAW SCALE for the SHADE
    mergeX=0.7 # cm
    mergeY=0.8 # cm
    height=fsizY*0.8 # cm
    width =0.2 # cm
    DD = set_scale_locate_bottom_right(fsizX,fsizY,mergeX,mergeY,height,width)
    command = "psscale" + ccol + DD + " -L " + GMT_M
    subprocess.run(command,shell=True)

    # DRAW UNIT with SCALE
    command = "pstext " + mmap + " -N " + " -G0 -N" + GMT_M
    p3 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)
    unit_size = 13  # cm
    mergin    = 1.0 # cm
    text_locate = set_text_locate_rightside_bottom(slon,elon,slat,elat,
                                             mergin,fsizX,unit_size)
    UNIT = "\(" + unit + "\)"
    text_line   = text_locate + UNIT
    command = "echo " + text_line
    subprocess.run(command,shell=True,stdout=p3.stdin)
    p3.stdin.close()

    # DRAW Jbound Line #####
    if JboundArea !="non" :
#        command = "Jbound --area " + JboundArea
        command = "Jbound2 --area " + JboundArea
        p1 = subprocess.Popen(command,shell=True,stdout=subprocess.PIPE)
        command = "psxy " + "-W4/100 -M "  + mmap + GMT_M 
        subprocess.run(command,shell=True,stdin=p1.stdout)
        p1.stdout.close()
    
    # DRAW Contour #####
    if cont != "non" :
        command = "grdcontour " + mmap + " "+str(cont)+" "+ cont_info + GMT_M 
        subprocess.run(command,shell=True)

    # DRAW Wind Vector #####
    if u != "non" and v != "non" :
        VEC_INFO = "-Q"+vec_info+" -S"+str(vec_per1cm)+" -I"+str(vec_INTV)
        U = " " + u + " "
        V = " " + v + " "
        command = "grdvector " + U + V + mmap + VEC_INFO + GMT_M
        subprocess.run(command,shell=True)
 
        command = "psxy " + mmap + " -N " + " -Sv" + vec_info + GMT_M
        p4 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)

        mergin = 0.4 # cm
        vec_locate = set_text_locate_top_right(slon,elon,slat,elat,
                                                 mergin,fsizY)        
        command = "echo " + vec_locate + " 180 0.25"
        subprocess.run(command,shell=True,stdout=p4.stdin)
        p4.stdin.close()

        mergin = 0.6 # cm
        text_size = 11 
        vecinfo_locate = set_text_locate_top_right(slon,elon,slat,elat,
                                                 mergin,fsizY,text_size)        
        text_line = vecinfo_locate + str(vec_per1cm*0.25) + "\(m/s\)"

        command = "pstext " + mmap + " -N " + " -G0  -N" + GMT_M
        p5 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)
        command = "echo " + text_line
        subprocess.run(command,shell=True,stdout=p5.stdin)
        p5.stdin.close()

    # DRAW Title text #####
    if title != "non" :
        command = "pstext " + mmap + " -N " + " -G0 -N" + GMT_M
        p2 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)

        text_size = 14  # cm
        mergin    = 0.2 # cm
        text_locate = set_text_locate_top_center(slon,elon,slat,elat,
                                                 mergin,fsizY,text_size)
        text_line = text_locate + title 

        command = "echo " + text_line
        subprocess.run(command,shell=True,stdout=p2.stdin)

        p2.stdin.close()


    # DRAW ADDITIONAL OBJECT ##########
    if obj_draw_command != "non":
        N  = len(obj_draw_data)
        D = len(obj_draw_data[0])
        command = obj_draw_command +" " +  mmap  + GMT_M
        p5 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)
        for n in range(N):
            lon = obj_draw_data[n][0]
            lat  = obj_draw_data[n][1]

            if D == 2 :
                command = "echo " + str(lon) + " " + str(lat) 

            elif D == 3: 
                aa  = obj_draw_data[n][2]
                command = "echo " + str(lon) + " " + str(lat)   + " " + str(aa)  

            elif D == 4: 
                aa  = obj_draw_data[n][2]
                bb  = obj_draw_data[n][3]
                cc   =  " " + str(aa) + " " + str(bb)  
                command = "echo " + str(lon) + " " + str(lat)   + " " + cc  

            subprocess.run(command,shell=True,stdout=p5.stdin)

        p5.stdin.close()

    # DRAW basemap
    command = "psbasemap " + mmap + Bbase + GMT_E
    subprocess.run(command,shell=True)

    if PNG != "non":
        make_PNG(fsizX,fsizY,PNG_yohakuX,PNG_yohakuY,PNG_DPI,PS,PNG)


def  draw_GMT_vertical(shade="non",shade_col="non",\
                       cont="non",cont_info="non",\
                       u="non",w="non",bottom=0.0,top=10.0, \
                       start=0.0,end=60.0,fsizX=14,fsizZ=7, PS="hoge.ps",\
                       PNG="hoge.png", bbase="a1000/5neSW",\
                       title="vertical cross seciton",unit="non",\
                       vec_INTV="5/0.5",vec_info="0.03c/0.11c/0.1c -W3 -G0",\
                       vec_per1cm=48,wamp=1.0,\
                       PNG_DPI = 100.0,PNG_yohakuX = 5,PNG_yohakuY = 2):
    
    if shade == "non" or shade_col=="non" :
        print ("shade information must be set")
        exit(1)

    default_GMT()
    
    rrr  = str(start) + "/" +str(end) + "/" + str(bottom) + "/" +str(top)
    proj       = " -JX" + str(fsizX) + "/" +str(fsizZ)
    rrange   = " -R" + rrr + " "
    ccol       = " -C" + shade_col + " "
    mmap    = rrange + proj + " -P "
    Bbase     = "-B" + bbase
    GMT_F    = " -K >" + PS
    GMT_M   = " -K -O >>" + PS
    GMT_E    = " -O >>" + PS

    if not os.path.exists(shade_col):
        print ("shade color palette was not given \n")
        auto_makecpt(shade,"no_green",shade_col,start,end,bottom,top)
        exit(1)

    
    # DRAW SHADE #####
    command = "grdimage " + shade + mmap +  ccol + "-Y1.5" + GMT_F
    subprocess.run(command,shell=True)
#    os.remove(shade)

    # DRAW SCALE for the SHADE
    mergeX=0.7 # cm
    mergeZ=0.8 # cm
    height=fsizZ*0.8 # cm
    width =0.2 # cm
    DD = set_scale_locate_bottom_right(fsizX,fsizZ,mergeX,mergeZ,height,width)
    command = "psscale" + ccol + DD + " -L " + GMT_M
    subprocess.run(command,shell=True)

    # DRAW UNIT with SCALE
    command = "pstext " + mmap + " -N " + " -G0 -N" + GMT_M
    p3 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)
    unit_size = 13  # cm
    mergin    = 1.0 # cm
    text_locate = set_text_locate_rightside_bottom(start,end,bottom,top,
                                                   mergin,fsizX,unit_size)
    UNIT = "\(" + unit + "\)"
    text_line   = text_locate + UNIT
    command = "echo " + text_line
    subprocess.run(command,shell=True,stdout=p3.stdin)
    p3.stdin.close()


    # DRAW Contour #####
    if cont != "non" :
        command = "grdcontour " + mmap + " "+str(cont)+" "+ cont_info + GMT_M 
        subprocess.run(command,shell=True)

    # DRAW Wind Vector #####
    if u != "non" and w != "non" :
        
        command = "grdmath " + w + " " + str(wamp) + " MUL = " + "wtemp.grd"
        subprocess.run(command,shell=True)

        VEC_INFO = "-Q"+vec_info+" -S"+str(vec_per1cm)+" -I"+vec_INTV
        U = " " + u + " "
        W = " " + "wtemp.grd" + " "
        command = "grdvector " + U + W + mmap + VEC_INFO + GMT_M
        subprocess.run(command,shell=True)
 
        command = "psxy " + mmap + " -N " + " -Sv" + vec_info + GMT_M
        p4 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)

        mergin = 0.4 # cm
        vec_locate = set_text_locate_top_right(start,end,bottom,top,
                                                 mergin,fsizZ)        

        draw_hori_wind_len = 0.25
        draw_ver_wind_len  = 0.25 
        command = "echo " + vec_locate + " 180 " + str(draw_hori_wind_len)
        subprocess.run(command,shell=True,stdout=p4.stdin)

        command = "echo " + vec_locate + " 90 " + str(draw_ver_wind_len)
        subprocess.run(command,shell=True,stdout=p4.stdin)

        p4.stdin.close()

        os.remove("wtemp.grd")

        ## draw reference wind vector ######
        mergin = 0.8 # cm
        text_size = 11 
        vecinfo_locate = set_text_locate_top_right(start,end,bottom,top,
                                                 mergin,fsizZ,text_size)        
        text_line1 = vecinfo_locate + "H:" + \
                    str(vec_per1cm*draw_hori_wind_len) + "\(m/s\)"


        mergin = 1.3 # cm
        text_size = 11 
        vecinfo_locate = set_text_locate_top_right(start,end,bottom,top,
                                                 mergin,fsizZ,text_size)        
        text_line2 = vecinfo_locate + "V:" + \
                    str(vec_per1cm*draw_ver_wind_len/wamp) + "\(m/s\)"


        command = "pstext " + mmap + " -N " + " -G0  -N" + GMT_M
        p5 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)

        command = "echo " + text_line1
        subprocess.run(command,shell=True,stdout=p5.stdin)

        command = "echo " + text_line2
        subprocess.run(command,shell=True,stdout=p5.stdin)

        p5.stdin.close()


    # DRAW Title text #####
    if title != "non" :
        command = "pstext " + mmap + " -N " + " -G0 -N" + GMT_M
        p2 = subprocess.Popen(command,shell=True,stdin=subprocess.PIPE)

        text_size = 14  # cm
        mergin    = 0.2 # cm
        text_locate = set_text_locate_top_center(start,end,bottom,top,
                                                 mergin,fsizZ,text_size)
        text_line = text_locate + title 

        command = "echo " + text_line
        subprocess.run(command,shell=True,stdout=p2.stdin)

        p2.stdin.close()

    
    # DRAW basemap
    command = "psbasemap " + mmap + Bbase + GMT_E
    subprocess.run(command,shell=True)
    

    if PNG != "non":
        make_PNG(fsizX,fsizZ,PNG_yohakuX,PNG_yohakuY,PNG_DPI,PS,PNG)    
