import subprocess
import os
import math
import numpy as np


from read_NC import read_NC
from read_NC import check_NC_type
from read_NC import get_missing_value


def cross_section_xy(LON,LAT,SS_EE,lon_per_1km=0.010966):
    
    dx = LON[2]-LON[1]
    dy = LAT[2]-LAT[1]
    dn = min(dx/lon_per_1km,dy/0.009)
    (lon0,lat0) = SS_EE[0]
    (lon1,lat1) = SS_EE[1]

    NX = len(LON)
    NY = len(LAT)

    distance = math.sqrt((lon1-lon0)*(lon1-lon0)/lon_per_1km/lon_per_1km \
                         +(lat1-lat0)*(lat1-lat0)/0.009/0.009)

    n = int(distance /dn)
    dis = distance/n

    unit_dx = (lon1-lon0)/n
    unit_dy = (lat1-lat0)/n
    n = n+1

    xy =[[0]*2 for i in range(n)]  ## initilization

    for i in range(n):
        x = lon0 + unit_dx * i
        y = lat0 + unit_dy * i
        xy[i] = (x,y)
        
    return xy


def get_vertical_profile(data3D,i0,j0,dx,dy,missing_value):

    NZ      = len(data3D)    
    profile =[0 for i in range(NZ)]  ## initilization

    for k in range(NZ):
        
        a = data3D[k][j0][i0]
        b = data3D[k][j0][i0+1]
        c = data3D[k][j0+1][i0]
        d = data3D[k][j0+1][i0+1]

        if a > missing_value+1 and  b > missing_value+1 and \
           c > missing_value+1 and  d > missing_value+1:
            var= (1-dy)*(a*(1-dx)+b*dx)+ dy* (c*(1-dx)+d*dx)
            profile[k] = var
        else:
            profile[k] = missing_value

    return profile


def vertical_interp(data_profile,zph_profile,LEV2,missing_value):

    NZ   = len(data_profile)
    NZ2 = len(LEV2)
    profile_new =[missing_value for i in range(NZ2)]  ## initilization

    for k2, zz2  in enumerate(LEV2):

        for k,zz in enumerate(zph_profile):

            if k==NZ :
                break

            if  zz2 >= zph_profile[k] and zz2 < zph_profile[k+1]:
                dz = (zz2 - zz)/(zph_profile[k+1]-zph_profile[k])

                if  data_profile[k] > missing_value+1 and \
                    data_profile[k+1] > missing_value+1 :
                    profile_new[k2] = (1-dz)*data_profile[k] + \
                                      dz*data_profile[k+1]
                else :
                    profile_new[k2] = missing_value
                
                break

    return profile_new


def make_grd_profile(profile,LEV,xy,missing_value,grdfile,lon_per_1km):

    M   = len(xy)
    NZ = len(LEV)

    pr1 = np.array(profile).T
    pr  = pr1.copy(order='C')
    outfile = open("ver.bin","wb")
    outfile.write(pr)
    outfile.close()

    (lon0,lat0) = xy[0]
    (lon1,lat1) = xy[M-1]

    dis = math.sqrt(((lon1-lon0)/lon_per_1km)**2.0 +((lat1-lat0)/0.009)**2.0)

    y0 = LEV[0]
    y1 = LEV[NZ-1]
    dy = LEV[1]-LEV[0]

    # unit change from meter to km
    y0 /= 1000.0
    y1 /= 1000.0
    dy /= 1000.0
    
    x0 = 0
    x1 = dis
    dx = dis/(M-1)

    cmd    = 'xyz2grd'
    inputF   = "ver.bin"
    RAN    = "-R" + str(x0) +"/" + str(x1) + "/" + str(y0) + "/" + str(y1)
    RES     = "-I" + str(dx) +"/" + str(dy)
    grdFF = "-G" + "tmp.grd"
    ZZZ     = "-ZBLd"
    cmd_list = [cmd,inputF,grdFF,RAN,RES,ZZZ]
    subprocess.run(cmd_list)
    os.remove("ver.bin")

    cmd     = 'grdclip'
    inputF  = "tmp.grd"
    outputF = "-G" + grdfile
    MIS  = missing_value+1
    VMIN    = "-Sb" + str(MIS) + "/NAN"
    cmd_list = [cmd,inputF,outputF,VMIN]
    subprocess.run(cmd_list)
    os.remove("tmp.grd")

def text_out_profile(profile,LEV,xy,outfile):
    
    outfile = open(outfile,"w")
    for m, (xx, yy)  in enumerate(xy):
        outfile.write('%s %s %s\n'  %(m,xx,yy))
        outfile.write("###########\n")
        for k, (zz)  in enumerate(LEV):
            outfile.write('%s %s %s\n' %(k,zz,profile[m][k]))
        outfile.write("###########\n")

    outfile.close()


def find_location(LON,xx):    
    i0 = -1
    dx = 0.0
    for i,lon in enumerate(LON):
        if lon > xx:
            dx = (LON[i] - xx)/(LON[1]-LON[0])
            i0 = i-1
            break
    return (i0,dx)

def calc_wind_comp_along_line(upro,vpro,theta,missing_value):
    M = len(upro)
    NZ= len(upro[0])
    along =[[0]*NZ for i in range(M)]  ## initilization    

    for m in range(M):
        for k in range(NZ):
            if upro[m][k] > missing_value+1:
                along[m][k] = upro[m][k]*math.cos(math.radians(theta))+\
                              vpro[m][k]*math.sin(math.radians(theta))
            else:
                along[m][k] = missing_value

    return  along

def vertical_cross_section(infile,varname,cross_section_Start_End,t,\
                           need_vertical_interp="NO",\
                           lon_per_1km=0.010966,textout="NO",\
                           u="non",v="non",w="non",addvar="non",\
                           uv_along_line="non"):
    
    dim_check = check_NC_type(infile,varname)
    if dim_check[1] != 1:
        print ("vertical coordinate is not found")
        exit(1)

    grdfile = varname + "_VER.grd"

    ##############################################
    data3D    = read_NC(infile,varname,t)
    LON       = read_NC(infile,"LON")
    LAT       = read_NC(infile,"LAT")
    LEV       = read_NC(infile,"ALT")
    NX        = len(LON)
    NY        = len(LAT)
    NZ        = len(LEV)
    missing_value = get_missing_value(infile,varname)

    if u!="non" and v!="non" and w!="non":
        U3D    = read_NC(infile,u,t)
        V3D    = read_NC(infile,v,t)
        W3D    = read_NC(infile,w,t)
        ugrd   = u + "_VER.grd"
        vgrd   = v + "_VER.grd"
        wgrd   = w + "_VER.grd"
    if addvar!="non":
        A3D    = read_NC(infile,addvar,t)
        agrd   = addvar + "_VER.grd"
    ##############################################

    ##############################################
    # setting of vertical interpolation 
    # for vertical coordinate with non-constant dz 
    if need_vertical_interp != "NO":
        dz_new     = LEV[1]-LEV[0]
        bottom_new = 0.0
        top_new    = int((LEV[NZ-1]-bottom_new)/dz_new)*dz_new +bottom_new
        NZ2  = int((top_new-bottom_new)/dz_new) + 1
        LEV2 =[bottom_new+dz_new*i  for i in range(NZ2)]  ## initilization
        ZPH  = read_NC(infile,"ZPH")
    ###############################################
    

    ##############################################
    # check if vertical cross section is inside of domain or not
    (lon0,lat0) = cross_section_Start_End[0]
    (lon1,lat1) = cross_section_Start_End[1]
    if lon0<LON[0] or lon1>LON[NX-1] or lat0<LAT[0] or lat1>LAT[NY-1]:
        print (" vertical cross section is out of domain")
        exit(1)

    if u!="non" and v!="non" and w!="non":
        dis_x = (lon1 - lon0)/lon_per_1km 
        dis_y = (lat1 - lat0)/0.009
        theta = math.degrees(math.atan2(dis_y,dis_x))
    ##############################################

    ##############################################
    # get lon,lat along the line of  vertical cross secion 
    # M is the number of grids along the line 
    xy  = cross_section_xy(LON,LAT,cross_section_Start_End,lon_per_1km)
    M   = len(xy)
    profile =[[0]*NZ for i in range(M)]  ## initilization

    if u!="non" and v!="non" and w!="non":
        upro =[[0]*NZ for i in range(M)]  ## initilization
        vpro =[[0]*NZ for i in range(M)]  ## initilization
        wpro =[[0]*NZ for i in range(M)]  ## initilization

    if addvar!="non":
        apro =[[0]*NZ for i in range(M)]  ## initilization
    ##############################################


    ##############################################
    if need_vertical_interp != "NO":
        interp_profile =[[0]*NZ2 for i in range(M)]  ## initilization
        zph_profile    =[[0]*NZ for i in range(M)]   ## initilization

        if u!="non" and v!="non" and w!="non":
            uintp =[[0]*NZ2 for i in range(M)]  ## initilization
            vintp =[[0]*NZ2 for i in range(M)]  ## initilization
            wintp =[[0]*NZ2 for i in range(M)]  ## initilization
        if addvar!="non":
            aintp =[[0]*NZ2 for i in range(M)]  ## initilization
    ##############################################


    ##############################################
    # get vertical profile at a grid point along the line
    for m, (xx, yy)  in enumerate(xy):

        (i0,dx)= find_location(LON,xx)
        (j0,dy)= find_location(LAT,yy)

        profile[m] = get_vertical_profile(data3D,i0,j0,dx,dy,missing_value)

        if u!="non" and v!="non" and w!="non":
            upro[m] = get_vertical_profile(U3D,i0,j0,dx,dy,missing_value)
            vpro[m] = get_vertical_profile(V3D,i0,j0,dx,dy,missing_value)
            wpro[m] = get_vertical_profile(W3D,i0,j0,dx,dy,missing_value)

        if addvar!="non":
            apro[m] = get_vertical_profile(A3D,i0,j0,dx,dy,missing_value)

        if need_vertical_interp != "NO":
            zph_profile[m] = get_vertical_profile(ZPH,i0,j0,dx,dy,\
                                                  missing_value)

            # get vertically interpotation 
            interp_profile[m] = vertical_interp(profile[m],zph_profile[m],\
                                                LEV2,missing_value)

            if u!="non" and v!="non" and w!="non":
                uintp[m] = vertical_interp(upro[m],zph_profile[m],\
                                                LEV2,missing_value)
                vintp[m] = vertical_interp(vpro[m],zph_profile[m],\
                                                LEV2,missing_value)
                wintp[m] = vertical_interp(wpro[m],zph_profile[m],\
                                                LEV2,missing_value)

            if addvar!="non":
                aintp[m] = vertical_interp(apro[m],zph_profile[m],\
                                                LEV2,missing_value)
    # end of loop for m ###########################

    # calcuate wind component parallel to the vertical cross section
    if uv_along_line !="non" and u!="non" and v!="non" :
        uv_along_grd = uv_along_line + "_VER.grd"

        if need_vertical_interp != "NO":
            uv_along= calc_wind_comp_along_line(uintp,vintp,theta,missing_value)
        else :
            uv_along= calc_wind_comp_along_line(upro,vpro,theta,missing_value)

    # output as GRDfile or Textfile ################
    if need_vertical_interp != "NO":

        make_grd_profile(interp_profile,LEV2,xy,missing_value,grdfile,\
                         lon_per_1km)

        if u!="non" and v!="non" and w!="non":
            make_grd_profile(uintp,LEV2,xy,missing_value,ugrd,lon_per_1km)
            make_grd_profile(vintp,LEV2,xy,missing_value,vgrd,lon_per_1km)
            make_grd_profile(wintp,LEV2,xy,missing_value,wgrd,lon_per_1km)

            if uv_along_line !="non":
                make_grd_profile(uv_along,LEV2,xy,missing_value,\
                                 uv_along_grd,lon_per_1km)

        if addvar!="non":
            make_grd_profile(aintp,LEV2,xy,missing_value,agrd,lon_per_1km)

        if textout != "NO":
            textF = varname + "_VER.txt"
            text_out_profile(interp_profile,LEV2,xy,textF)
            if u!="non" and v!="non" and w!="non":
                textF = u + "_VER.txt"
                text_out_profile(uintp,LEV2,xy,textF)
                textF = v + "_VER.txt"
                text_out_profile(vintp,LEV2,xy,textF)
                textF = w + "_VER.txt"
                text_out_profile(wintp,LEV2,xy,textF)

                if uv_along_line !="non":
                    textF = uv_along_line + "_VER.txt"
                    text_out_profile(uv_along,LEV2,xy,textF)

            if addvar!="non":
                testF = addvar + "_VER.txt"
                text_out_profile(aintp,LEV2,xy,testF)
 
    else:

        make_grd_profile(profile,LEV,xy,missing_value,grdfile,\
                         lon_per_1km)

        if u!="non" and v!="non" and w!="non":
            make_grd_profile(upro,LEV,xy,missing_value,ugrd,lon_per_1km)
            make_grd_profile(vpro,LEV,xy,missing_value,vgrd,lon_per_1km)
            make_grd_profile(wpro,LEV,xy,missing_value,wgrd,lon_per_1km)

            if uv_along_line !="non":
                make_grd_profile(uv_along,LEV,xy,missing_value,\
                                 uv_along_grd,lon_per_1km)

        if addvar!="non":
            make_grd_profile(wpro,LEV,xy,missing_value,agrd,lon_per_1km)

        if textout != "NO":
            textF = varname + "_VER.txt"
            text_out_profile(profile,LEV,xy,textF)

            if u!="non" and v!="non" and w!="non":
                textF = u + "_VER.txt"
                text_out_profile(upro,LEV,xy,textF)
                textF = v + "_VER.txt"
                text_out_profile(vpro,LEV,xy,textF)
                textF = w + "_VER.txt"
                text_out_profile(wpro,LEV,xy,textF)

                if uv_along_line !="non":
                    textF = uv_along_line + "_VER.txt"
                    text_out_profile(uv_along,LEV,xy,textF)

            if addvar!="non":
                testF = addvar + "_VER.txt"
                text_out_profile(apro,LEV,xy,testF)

def change_input_ver_cross(slon,slat,dis,rot,lon_per_1km):

    elon = slon + dis * lon_per_1km * math.cos(math.radians(rot))  
    elat  = slat +  dis * 0.009 * math.sin(math.radians(rot))  
    
    return ([[slon,slat],[elon,elat]])

